// Türklingel mit 100 Melodien M. Baudisch 12/2008
// mit Änderungen vom 08.01.09 Michael S.

#include <inttypes.h>		// uint8_t usw.
#include <avr/io.h>
#include <avr/interrupt.h>	// Interrupts
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <util/delay.h>		// definiert _delay_ms und _delay_loop_2

#include "lied.h"          // Melodien, Offsets und Tonhöhen

#define   SPEAKER         1 << PB1
#define   SOUNDON         TCCR1=(1<<CTC1)|(1<<COM1A0)|(1<<CS10)|(1<<CS11)|(1<<CS12)
#define   SOUNDOFF        TCCR1=0

//TCCR1 = (1<<CTC1) |     Timer wird nach erreichen des Vergleichswertes
//                        OCR1A rückgesetzt OCR1A -> Frequenz
//        (1<<COM1A0) |   Output (OC1A/B1) wird getoggled
//        (1<<CS10)|(1<<CS11)|(1<<CS12);  //Vorteiler 64

volatile int16_t soundtime;
extern   uint8_t buffer[];

uint16_t liedcount;
uint16_t liedende;
uint8_t 	liednr;
uint8_t 	speed;
uint8_t 	dauer;
uint8_t 	hoehe;
uint8_t 	note;

// ----------------------------------------------------------------------------
void klingel_init(void)
{
														// Timer 0 initialisieren
TCCR0B=(1<<CS02)|(0<<CS01)|(1<<CS00); 		// Vorteiler 1024
TCCR0A=(1<<WGM01); 								// CTC Mode

TIMSK = (1 << OCIE0A); 							// Interrupts auf Comp Wert A zulassen
OCR0A = 155;										// ergibt ca. 20ms bzw. 50Hz
}

// ----------------------------------------------------------------------------
// Interrupt Zeitsteuerung
ISR(TIM0_COMPA_vect)
{
if (soundtime) soundtime--;
}

// -----------------------------------------------------------------------------
void klingel(uint8_t liednr)
{
// ein Array mit Pointern auf die Liedarrays
// die Lieder selbst sind in lied.h definiert
uint8_t *lieder[] = {L1 , L2 ,  L3,  L4,  L5,  L6,  L7,  L8,  L9, L10,
							L11, L12, L13, L14, L15, L16, L17, L18, L19, L20,
							L21, L22, L23, L24, L25, L26, L27, L28, L29, L30,
							L31, L32, L33, L34, L35, L36, L37, L38, L39, L40,
							L41, L42, L43, L44, L45, L46, L47, L48, L49, L50,
							L51, L52, L53, L54, L55, L56, L57, L58, L59, L60,
							L61, L62, L63, L64, L65, L66, L67, L68, L69, L70,
							L71, L72, L73, L74, L75, L76, L77, L78, L79, L80,
							L81, L82, L83, L84, L85, L86, L87, L88, L89, L90,
							L91, L92, L93, L94, L95, L96, L97, L98, L99, L100} ;
							
uint8_t *ptr = 0;									// diesem Pointer wird das Lied zugewiesen
speed		= 0;
soundtime= 0;

if (liednr > 101) liednr = 1;					// Nr. 101 kommt durch !
if (!(liednr))    liednr = 1;					// wenn == 0

while(1)
	{
	if (liednr) 									// wenn Melodie aktiv
		{
		if (!soundtime) 							// wenn kein Ton oder Pause läuft
			{  										// keine Tonzeitdauer aktiv
			SOUNDOFF;     							// Tonwiedergabe aus
			if (!speed) 							// wir sind am Liedanfang
            {
				if (!(liednr == 101))			// dann steht das Lied im Array lieder[]
					{
					ptr = lieder[liednr - 1]; 	// den Pointer auf das Lied holen
					liedende = pgm_read_byte(&ptr[0]); // Werte aus Flash auslesen
					speed    = pgm_read_byte(&ptr[1]);
					}

				else									// dann steht das Lied in buffer[]
					{
					ptr = buffer;					// aus usi_klingel.c
					liedende = buffer[0];		// Werte aus SRAM auslesen
					speed    = buffer[1];
					}

				DDRB  |= SPEAKER; 				// Speaker an
            liedcount = 2;						// der erste Ton
				}

			else 										// mitten im lied
				{
            _delay_ms(20); 					// um Töne zu trennen
														// Ton ausgeben
            if (!(liednr == 101))			// dann stehen die Noten im Flash
					{
					note=pgm_read_byte(&ptr[liedcount]); // Note holen
					}
				else									// dann stehen die Noten in buffer[]
					{
					note = buffer[liedcount];
					}
				hoehe=note & 0x1f; 				// Tonhöhe ausmaskieren (5 Bit)
            dauer= zeit[(note >> 5)]; 		// 5 Bit nach rechts schieben und Zeit holen
            if (hoehe) 							// wenn keine Pause
					{
	            OCR1C = pgm_read_byte(&ton[hoehe-1]);
               SOUNDON;
               }

            cli();     							// da Variable vom Interrupt 
            soundtime=dauer*speed;			// während der Zuweisung geändert werden könnte
            sei();

            liedcount++;

            if (liedcount>liedende)       // Lied zu Ende
					{
					speed =0;
					liednr=0;
					}
				}
			} 											// (!soundtime)

		} 												// liednr

	else
		{

		SOUNDOFF; 									// falls noch Ton an

		DDRB  &= ~(SPEAKER); 					// Speaker Port hochohmig
		break;										// Rücksprung
		}
	}													// for
} 														// main

